@@ -1,5 +1,5 @@
 import os
-import logging
+from logging import Logger, INFO
 import json
 from typing import Dict, List
 import urllib.request
@@ -17,7 +17,7 @@ class Config:
 
     allowed_extensions = ['yml', 'toml']
 
-    def __init__(self, profiles: List[str] = ['default'], locations: List[str] = ['./', './config/']) -> None:
+    def __init__(self, profiles: List[str] = ['default'], locations: List[str] = ['./', './config/'], logger: Logger = None, log_level=INFO) -> None:
         """
         Parameters
         ----------
@@ -25,22 +25,31 @@ class Config:
             List of all required profiles. Default value is ['default']
         locations : List[str] 
             List of paths where this class will scan for configuration files
+        logger: Logger
+            Inject optional logger
+        log_level: str
+            Log level for default logger if no logger provided
         """
+        # init logger if not provided
+        if logger == None:
+            logger = Logger('ecmind_spring_config', log_level)
         
+        self.logger = logger
+
         # init empty configuration
         self._config: Dict[str, any] = {}
 
         # load environment into config
-        logging.debug('Parse environment')
+        self.logger.debug('Parse environment')
         for key in os.environ.keys():
-            self.put(key, os.environ.get(key))
+            self.put(self.to_camel_case(key), os.environ.get(key))
 
         # check if key spring.profiles.active is set and load this profiles. 
         # The environment parameter will overwrite the function parameter
         if self.get('spring.profiles.active') == None:
             self.put('spring.profiles.active', profiles)
 
-        logging.debug(f'Active profiles are {self.get_profiles()}')
+        self.logger.debug(f'Active profiles are {self.get_profiles()}')
 
         # check if key spring.config.location is set and load this locations. 
         # The environment parameter will overwrite the function parameter       
@@ -48,28 +57,33 @@ class Config:
             self.put('spring.config.location', locations)
 
         locations = self.get_locations()
-        logging.debug(f'Config locations are {locations}')
+        self.logger.debug(f'Config locations are {locations}')
 
         # check each location
         for location in locations:
             # load config directly if location is a file path
             if os.path.isfile(location):
-                logging.debug(f'Parse config {locations}')
+                self.logger.debug(f'Parse config {locations}')
                 self._parse_config_file(location)
             # check for expected config files if location is a path
             elif os.path.isdir(location):
                 for extension in Config.allowed_extensions:
                     filepath = location + f"/application.{extension}"
                     if os.path.isfile(filepath):
-                        logging.debug(f'Parse config {filepath}')
+                        self.logger.debug(f'Parse config {filepath}')
                         self._parse_config_file(filepath)
                     #check for each profile name if a config file exists
                     for profile in self.get_profiles():
                         filepath = location + f"/application-{profile}.{extension}"
                         if os.path.isfile(filepath):
-                            logging.debug(f'Parse config {filepath}')
+                            self.logger.debug(f'Parse config {filepath}')
                             self._parse_config_file(filepath)
         
+        # Overwrite file config with environment
+        self.logger.debug('Reparse environment')
+        for key in os.environ.keys():
+            self.put(self.to_camel_case(key), os.environ.get(key))
+
         # load from spring cloud config server if possible 
         self.load_cloud_config()
 
@@ -102,11 +116,15 @@ class Config:
         key = key.lower()
 
         item = self._config
-        try:
-            for key_part in key.split("."):
-                item = item[key_part]
-        except KeyError:
-            return default
+        for key_part in key.split("."):
+            found = False
+            for item_key in item.keys():
+                if item_key.lower() == key_part:
+                    item = item[item_key]
+                    found = True
+                    break
+            if not found: 
+                return default
 
         item = self.replace_variables(item)
 
@@ -142,8 +160,6 @@ class Config:
         if data == None:
             data = self._config
         
-        key = key.lower()
-
         if "." in key:
             key, rest = key.split(".", 1)
             if key not in data:
@@ -179,11 +195,9 @@ class Config:
         """
         if filepath.endswith('yml'):
             config = yaml.load(open(filepath), Loader=yaml.FullLoader)
-            config = self._make_lowercase(config)
             self._config = always_merger.merge(self._config, config)
         elif filepath.endswith('toml'):
             config = toml.load(filepath)
-            config = self._make_lowercase(config)
             self._config = always_merger.merge(self._config, config)
 
     def load_cloud_config(self):
@@ -204,46 +218,28 @@ class Config:
             return
 
         if app_name == None:
-            raise ValueError('Key spring.application.name is required')
-
-        logging.info(
-            f"Try to load configuration from spring cloud config server at {self['spring.cloud.config.uri']}")
+            raise ValueError('If spring.cloud.config.uri is set spring.application.name is required')
 
         if label:
             address = f'{uri}/{label}/{app_name}-{profiles}.json'
         else:
             address = f'{uri}/{app_name}-{profiles}.json'
 
+        self.logger.info(
+            f"Try to load configuration from spring cloud config server at {address}")
+
         try:
             with urllib.request.urlopen(address) as response:
                 config = json.loads(response.read())
                 # Keys to lower case
-                config = self._make_lowercase(config)
+                #config = self._make_lowercase(config)
 
                 self._config = always_merger.merge(self._config, config)
 
         except urllib.error.URLError as e:
-            logging.error(e.reason)
+            self.logger.error(e.reason)
             raise e
 
-
-
-    def _make_lowercase(self, obj):
-        if isinstance(obj, dict):
-            ret = {}
-            for k,v in obj.items():
-                ret[k.lower()] = self._make_lowercase(v)
-            return ret
-        elif isinstance(obj, str):
-            return obj
-        elif hasattr(obj,'__iter__'):
-            ret = []
-            for item in obj:
-                ret.append(self._make_lowercase(item))
-            return ret
-        else:
-            return obj
-
     def replace_variables(self, obj):
         if isinstance(obj, dict):
             ret = {}
@@ -274,5 +270,7 @@ class Config:
         else:
             return obj
 
-
+    def to_camel_case(self, snake_str: str):
+        components = snake_str.split('_')
+        return components[0] + ''.join(x.title() for x in components[1:])
                     
\ No newline at end of file
